@@ -9,7 +9,6 @@ class ClientConfig:
     Used to hold the Client config.
     """
     username: str
-    events: list
     headers: dict
     cookies: dict
 
@@ -21,7 +20,7 @@ class Client:
     client_config = None
 
     @classmethod
-    def set_client_config(cls, username: str, sid_cookie: str, csrf_cookie: str) -> None:
+    def set_client_config(cls, *, username: str, sid_cookie: str, csrf_cookie: str) -> None:
         from pythontextnow.api.TextNowAPI import TextNowAPI
         text_now_api = TextNowAPI()
         cookies = {
@@ -34,7 +33,6 @@ class Client:
         }
 
         client_config = ClientConfig(username=username,
-                                     events=list(),
                                      headers=headers,
                                      cookies=cookies)
         cls.client_config = client_config
