@@ -2,6 +2,7 @@ import json
 import urllib
 from datetime import datetime
 from typing import Optional
+from urllib import parse
 from urllib.parse import quote
 
 import requests
@@ -17,10 +18,13 @@ from pythontextnow.util.ConfigReader import ConfigReader
 class TextNowAPI:
     def __init__(self):
         self.__BASE_URL = ConfigReader.get("api", "textnow_base_url")
+        self.__VERSION = ConfigReader.get("api", "version")
         self.__API_ROUTE = ConfigReader.get("api", "api_route")
+        self.__ATTACHMENT_URL_ROUTE = ConfigReader.get("api", "attachment_url_route")
         self.__CONVERSATIONS_ROUTE = ConfigReader.get("api", "conversations_route")
         self.__MESSAGES_ROUTE = ConfigReader.get("api", "messages_route")
         self.__MESSAGING_ROUTE = ConfigReader.get("api", "messaging_route")
+        self.__SEND_ATTACHMENT_ROUTE = ConfigReader.get("api", "send_attachment_route")
         self.__USERS_ROUTE = ConfigReader.get("api", "users_route")
 
         self.__MAX_MESSAGE_RESPONSE_SIZE = 30
@@ -35,12 +39,12 @@ class TextNowAPI:
 
         response.raise_for_status()
 
-        resp = response.text
+        response_text = response.text
         needle = 'csrf-token" content="'
-        needle_index = resp.find(needle)
+        needle_index = response_text.find(needle)
         token_start = needle_index + len(needle)
-        token_end = resp.find('"', token_start)
-        csrf_token = resp[token_start:token_end]
+        token_end = response_text.find('"', token_start)
+        csrf_token = response_text[token_start:token_end]
         return csrf_token
 
     def send_message(self, *, message: str, send_to: str) -> None:
@@ -49,7 +53,7 @@ class TextNowAPI:
                      "message": message,
                      "read": ReadStatus.READ.value,
                      "message_direction": MessageDirection.OUTGOING.value,
-                     "message_type": MessageType.MULTIMEDIA.value,
+                     "message_type": MessageType.TEXT.value,
                      "from_name": self.__client_config.username,
                      "has_video": False,
                      "new": True,
@@ -93,7 +97,7 @@ class TextNowAPI:
             cookies=self.__client_config.cookies)
         response.raise_for_status()
 
-        message_dicts = json.loads(response.content)["messages"]
+        message_dicts = response.json()["messages"]
 
         all_messages = list()
         # sort into Text and MultiMedia messages
@@ -101,7 +105,7 @@ class TextNowAPI:
             message_type = MessageType.from_value(message_dict["message_type"])
             if message_type == MessageType.TEXT:
                 all_messages.append(TextMessage.from_dict(message_dict))
-            elif message_type == MessageType.MULTIMEDIA:
+            elif message_type in (MessageType.IMAGE, MessageType.VIDEO):
                 all_messages.append(MultiMediaMessage.from_dict(message_dict))
         return all_messages
 
@@ -117,11 +121,11 @@ class TextNowAPI:
 
         data = {"read": True}
 
-        response = requests.post(url,
-                                 params=params,
-                                 data=data,
-                                 cookies=self.__client_config.cookies,
-                                 headers=self.__client_config.headers)
+        response = requests.patch(url,
+                                  params=params,
+                                  data=data,
+                                  cookies=self.__client_config.cookies,
+                                  headers=self.__client_config.headers)
         response.raise_for_status()
 
     def delete_message(self, *, message: Optional[Message] = None, message_id: Optional[str] = None) -> None:
@@ -138,3 +142,67 @@ class TextNowAPI:
                                    cookies=self.__client_config.cookies,
                                    headers=self.__client_config.headers)
         response.raise_for_status()
+
+    def get_attachment_url(self, *, message_type: MessageType) -> str:
+        """
+        Gets the URL that a file can be uploaded to.
+        """
+        url = f"{self.__BASE_URL}{self.__API_ROUTE}/{self.__VERSION}{self.__ATTACHMENT_URL_ROUTE}"
+        params = {"message_type": message_type.value}
+        url_with_params = f"{url}?{parse.urlencode(params)}"
+
+        response = requests.get(
+            url_with_params,
+            headers=self.__client_config.headers,
+            cookies=self.__client_config.cookies)
+        response.raise_for_status()
+
+        return response.json()["result"]
+
+    def upload_raw_media(self, *, attachment_url: str, raw_media: bytes, media_type: str) -> None:
+        """
+        Uploads the given raw_media to the given URL.
+        """
+        headers = {
+            'accept': '*/*',
+            'content-type': media_type,
+            'accept-language': 'en-US,en;q=0.9',
+            "mode": "cors",
+            "method": "PUT",
+            "credentials": 'omit'
+        }
+
+        response = requests.put(attachment_url,
+                                data=raw_media,
+                                headers=headers,
+                                cookies=self.__client_config.cookies)
+        response.raise_for_status()
+
+    def send_attachment(self, *,
+                        conversation_phone_number: str,
+                        message_type: MessageType,
+                        file_type: str,
+                        is_video: bool,
+                        attachment_url: str) -> None:
+
+        data = {
+            "contact_value": conversation_phone_number,
+            "contact_type": ContactType.MEDIA.value,
+            "read": 1,
+            "message_direction": MessageDirection.OUTGOING.value,
+            "message_type": message_type.value,
+            "from_name": self.__client_config.username,
+            "has_video": is_video,
+            "new": True,
+            "date": datetime.now().isoformat(),
+            "attachment_url": attachment_url,
+            "media_type": file_type
+        }
+
+        url = f"{self.__BASE_URL}{self.__API_ROUTE}/{self.__VERSION}{self.__SEND_ATTACHMENT_ROUTE}"
+
+        response = requests.post(url,
+                                 data=data,
+                                 headers=self.__client_config.headers,
+                                 cookies=self.__client_config.cookies)
+        response.raise_for_status()
